<?php

namespace SM\Factory;

use Doctrine\ORM\EntityManager;
use \Teknisa\Model\Traits\TQuery;

class Sm00005000TabParametrizacoesIndicesPadroesFactory
{
    use TQuery;

    protected $connection;

    protected $entityManager;

    public function __construct(EntityManager $entityManager)
    {
        $this->connection = $entityManager->getConnection();
        $this->entityManager = $entityManager;
    }

    public function getIndicesPadroes()
    {
        $sql = "SELECT FCT_INDICESPADROES.NRINDICESPADROES,
                        FCT_INDICESPADROES.CDTPINDICE,
                        FCT_INDICESPADROES.IDPERVALOR,
                        FCT_INDICESPADROES.CDINDICE,
                        INDICE.NMINDICE,
                        FCT_INDICESPADROES.NRPERCENTINDICE,
                        FCT_INDICESPADROES.IDATIVO,
                        FCT_INDICESPADROES.NRORG,
                        FCT_INDICESPADROES.DTINCLUSAO,
                        FCT_INDICESPADROES.NRORGINCLUSAO,
                        FCT_INDICESPADROES.CDOPERINCLUSAO,
                        FCT_INDICESPADROES.DTULTATU,
                        FCT_INDICESPADROES.NRORGULTATU,
                        FCT_INDICESPADROES.CDOPERULTATU
                FROM FCT_INDICESPADROES
                JOIN INDICE ON INDICE.CDTPINDICE = FCT_INDICESPADROES.CDTPINDICE
                AND INDICE.IDPERVALOR = FCT_INDICESPADROES.IDPERVALOR
                AND INDICE.CDINDICE = FCT_INDICESPADROES.CDINDICE
                AND INDICE.NRORG = FCT_INDICESPADROES.NRORG";

        return $this->createAndExecuteQueryAndBuildResponseData($this->entityManager, $sql);
    }

    public function getIndiceFinanceiro()
    {
        $sql = "SELECT INDICE.CDTPINDICE,
                    INDICE.IDPERVALOR,
                    INDICE.CDINDICE,
                    INDICE.SGINDICE,
                    INDICE.NMINDICE,
                    MOVINDICE.DTREFIND,
                    MOVINDICE.VRINDICE,
                    INDICE.NRORG
                FROM INDICE
                JOIN MOVINDICE ON INDICE.CDTPINDICE = MOVINDICE.CDTPINDICE
                AND INDICE.IDPERVALOR = MOVINDICE.IDPERVALOR
                AND INDICE.CDINDICE = MOVINDICE.CDINDICE
                AND INDICE.NRORG = MOVINDICE.NRORG JOIN
                (SELECT CDTPINDICE, IDPERVALOR, CDINDICE, NRORG, MAX(DTREFIND) AS DTREFIND
                FROM MOVINDICE
                GROUP BY CDTPINDICE, IDPERVALOR, CDINDICE, NRORG) DTIND ON MOVINDICE.CDTPINDICE = DTIND.CDTPINDICE
                AND MOVINDICE.IDPERVALOR = DTIND.IDPERVALOR
                AND MOVINDICE.CDINDICE = DTIND.CDINDICE
                AND MOVINDICE.NRORG = DTIND.NRORG
                AND MOVINDICE.DTREFIND = DTIND.DTREFIND";

        return $this->createAndExecuteQueryAndBuildResponseData($this->entityManager, $sql);
    }

    public function validaRegistrosDuplicados($params)
    {

        $sql = "SELECT COUNT (*) AS QTD 
                FROM FCT_INDICESPADROES
                WHERE CDINDICE      = :CDINDICE
                AND NRINDICESPADROES <> NVL(:NRINDICESPADROES, -1)";

        return $this->connection->executeQuery($sql, $params)->fetchAllAssociative();
    }

    public function updateIndiceFinanceiroPadrao($params)
    {
        $sql = "UPDATE FCT_INDICESPADROES
                   SET CDINDICE         = :CDINDICE,
                       CDTPINDICE       = :CDTPINDICE,
                       IDPERVALOR       = :IDPERVALOR, 
                       NRPERCENTINDICE = :NRPERCENTINDICE,
                       NRORG          = :NRORG,
                       DTULTATU       = SYSDATE,
                       CDOPERULTATU   = :CDOPERULTATU,
                       IDATIVO        = :IDATIVO
                 WHERE NRINDICESPADROES = :NRINDICESPADROES";

        $this->connection->executeStatement($sql, $params);
    }

    public function insertIndiceFinanceiroPadrao($params)
    {
        $sql = "INSERT INTO FCT_INDICESPADROES(
                    NRINDICESPADROES, CDINDICE, CDTPINDICE, IDPERVALOR, NRPERCENTINDICE, NRORG, DTINCLUSAO, CDOPERINCLUSAO, IDATIVO)
                VALUES(
                    :NRINDICESPADROES, :CDINDICE, :CDTPINDICE, :IDPERVALOR, :NRPERCENTINDICE, :NRORG, SYSDATE, :CDOPERINCLUSAO, :IDATIVO)";

        $this->connection->executeStatement($sql, $params);
    }

    public function deleteParametrosIndice($params)
    {
        $sql = "DELETE FROM FCT_INDICESPADROES WHERE NRINDICESPADROES = :NRINDICESPADROES";
        $this->connection->executeStatement($sql, $params);
    }
}
