<?php

namespace SM\Factory;

use Doctrine\ORM\EntityManager;
use SM\Traits\TQuery;

class Sm00070000HierarquiaOperadoresFactory
{
    use TQuery;

    protected $connection;

    protected $entityManager;

    public function __construct(EntityManager $entityManager)
    {
        $this->connection = $entityManager->getConnection();
        $this->entityManager = $entityManager;
    }

    public function getHierarquiaOperadores()
    {
        $sql = "SELECT HIERARQUIA.*,
                        NMFILIAL,
                        HIERARQUIAPAI.DSHIERARQUIAOPER AS DSHIERARQUIAOPERPAI,
                        DECODE(PAIS.PAI, NULL, 'N', 'S') AS PAI
                FROM CRM_HIERARQUIAOPER HIERARQUIA
                LEFT JOIN CRM_HIERARQUIAOPER HIERARQUIAPAI ON HIERARQUIAPAI.NRHIERARQUIAOPER = HIERARQUIA.NRHIERARQUIAPAI
                    AND HIERARQUIAPAI.NRORG = HIERARQUIA.NRORG
                LEFT JOIN FILIAL ON HIERARQUIA.CDFILIAL = FILIAL.CDFILIAL
                    AND HIERARQUIA.NRORG = FILIAL.NRORG
                LEFT JOIN (SELECT DISTINCT NRHIERARQUIAPAI AS PAI 
                                FROM CRM_HIERARQUIAOPER 
                            WHERE NRHIERARQUIAPAI IS NOT NULL) PAIS
                    ON HIERARQUIA.NRHIERARQUIAOPER = PAIS.PAI";

        return $this->createAndExecuteQueryAndBuildResponseData($this->entityManager, $sql);
    }

    public function getOperadorHierarquia()
    {
        $sql = "SELECT HIERARQUIA.CDOPERADOR,
                       HIERARQUIA.NRHIERARQUIAPAI,
                       HIERARQUIAPAI.DSHIERARQUIAOPER AS DSHIERARQUIAOPERPAI
                 FROM CRM_HIERARQUIAOPER HIERARQUIA
                 LEFT JOIN CRM_HIERARQUIAOPER HIERARQUIAPAI ON HIERARQUIAPAI.NRHIERARQUIAOPER = HIERARQUIA.NRHIERARQUIAPAI
                 AND HIERARQUIAPAI.NRORG = HIERARQUIA.NRORG";

        return $this->createAndExecuteQueryAndBuildResponseData($this->entityManager, $sql);
    }

    public function insertHierarquia(array $params)
    {
        $sql = "INSERT INTO CRM_HIERARQUIAOPER
                (
                    NRHIERARQUIAOPER,     DSHIERARQUIAOPER,   NRHIERARQUIAPAI,
                    IDOPERMASTER,         NRORG,              CDOPERADOR,         
                    DTINCLUSAO,           CDOPERINCLUSAO,     NRORGINCLUSAO,  
                    IDATIVO,              CDFILIAL
                )
                VALUES 
                (
                    :NRHIERARQUIAOPER,    :DSHIERARQUIAOPER,  :NRHIERARQUIAPAI,
                    :IDOPERMASTER,        :NRORG,             :CDOPERADOR,       
                    SYSDATE,              :CDOPERINCLUSAO,    :NRORGINCLUSAO,
                    :IDATIVO,             :CDFILIAL
                )";
        $this->connection->executeStatement($sql, $params);
    }

    public function updateHierarquia(array $params)
    {
        $sql = "UPDATE CRM_HIERARQUIAOPER 
                SET DSHIERARQUIAOPER    = :DSHIERARQUIAOPER, 
                NRHIERARQUIAPAI         = :NRHIERARQUIAPAI,
                IDOPERMASTER            = :IDOPERMASTER,
                NRORG                   = :NRORG, 
                CDOPERADOR              = :CDOPERADOR,
                DTULTATU                = SYSDATE,
                CDOPERULTATU            = :CDOPERULTATU,
                NRORGULTATU             = :NRORGULTATU,
                IDATIVO                 = :IDATIVO,
                CDFILIAL                = :CDFILIAL
                WHERE NRHIERARQUIAOPER  = :NRHIERARQUIAOPER";

        $this->connection->executeStatement($sql, $params);
    }

    public function deleteHierarquia($params)
    {
        $sql = "DELETE FROM CRM_HIERARQUIAOPER WHERE NRHIERARQUIAOPER = :NRHIERARQUIAOPER";
        $this->connection->executeStatement($sql, $params);
    }
}
