import { Utilities } from '@/util/Utilities';
import {
    Modal, ModalService, Select, Text, Form, LoadingService, DialogService,
} from '@zeedhi/common';
import { Http, Metadata } from '@zeedhi/core';
import { TekGrid } from '@zeedhi/teknisa-components-common';

export class EscopoFaturamentoTabTabelaPrecoController {
    private basicModal!: Modal;

    public async exibirModalTabelaPreco() {
        this.basicModal = await ModalService.createFromJson('escopoFaturamento/modal/escopoFaturamentoModalTabelaPreco', true);
        this.basicModal.show();
    }

    public fecharModalTabelaPreco() {
        this.basicModal.hide();
        this.basicModal?.destroy();
    }

    public async abrirModalEdicaoTabelaPreco({ row } : any) {
        await this.exibirModalTabelaPreco();
        Metadata.getInstance<Text>('titleText').text = 'Editar Tabela de Preço';
        Metadata.getInstance<Select>('FORMTABPREC-NMMOEDA').value = row.NRMOEDA;
        Metadata.getInstance<Select>('FORMTABPREC-DSPRECBASESERV').value = row.CDPRECBASESERV;
        Metadata.getInstance<Select>('FORMTABPREC-DSPRECOBASE').value = row.CDPRECBASEVEND;
    }

    public async abrirModalAdicionaTabelaPreco() {
        await this.exibirModalTabelaPreco();
        Metadata.getInstance<Text>('titleText').text = 'Nova Tabela de Preço';
    }

    public saveTabelaPreco() {
        const formTabelaPreco = Metadata.getInstance<Form>('form_modal_tabela_preco');
        const gridEscopoFaturamento = Metadata.getInstance<TekGrid>('tek-grid-escopo-faturamento_grid');
        const gridTabelaPreco = Metadata.getInstance<TekGrid>('basic-tek-grid-tab-tabela-preco');
        const params = {
            NRTABPRECESCOPOFAT: gridTabelaPreco.datasource.currentRow.NRTABPRECESCOPOFAT,
            NRTIPOORCAMENTO: gridEscopoFaturamento.datasource.currentRow.NRTIPOORCAMENTO,
            NRMOEDA: Metadata.getInstance<Select>('FORMTABPREC-NMMOEDA').value,
            CDPRECBASESERV: Metadata.getInstance<Select>('FORMTABPREC-DSPRECBASESERV').value,
            CDPRECBASEVEND: Metadata.getInstance<Select>('FORMTABPREC-DSPRECOBASE').value,
        };

        if (Utilities.validateForm(formTabelaPreco)) {
            Http.post('/saveTabelaPreco', params).then(() => {
                gridTabelaPreco.datasource.get();
                this.fecharModalTabelaPreco();
                Utilities.showSuccessMessage();
            }).catch((error: any) => {
                if (error.response.data.error === 'EXISTE') {
                    Utilities.showDialogMessage('error', 'Operação Bloqueada!', 'Já existe uma tabela de preço cadastrada para a moeda informada.');
                } else {
                    Utilities.showErrorMessage(error);
                }
            }).finally(() => {
                LoadingService.hide();
            });
        }
    }

    public showConfirmDeleteMessageTabelaPreco({ event }: any) {
        event!.preventDefault();
        DialogService.show({
            name: 'buttons-dialog-documento-contrato',
            type: 'warning',
            title: 'Excluir Registro',
            text: 'Deseja realmente excluir todos os registros selecionados?',
            buttons: [
                {
                    name: 'NAO',
                    component: 'ZdButton',
                    label: 'Não',
                    outline: true,
                    events: {
                        click: () => {
                            DialogService.hide();
                        },
                    },
                },
                {
                    name: 'SIM',
                    component: 'ZdButton',
                    label: 'Sim',
                    events: {
                        click: () => {
                            this.deleteTabPrecEscopoFat();
                            DialogService.hide();
                        },
                    },
                },
            ],
        });
    }

    public deleteTabPrecEscopoFat() {
        const grid = Metadata.getInstance<TekGrid>('basic-tek-grid-tab-tabela-preco');
        LoadingService.show();
        Http.post('/deleteTabPrecEscopoFat', {
            NRS: grid.selectedRows.map((element: any) => element.NRTABPRECESCOPOFAT),
        }).then(() => {
            grid.selectedRows = [];
            grid.reload();
            Utilities.showSuccessMessage();
        }).catch((error) => {
            Utilities.showErrorMessage(error);
        }).finally(() => {
            LoadingService.hide();
        });
    }
}
