import {
    Metadata, Http,
} from '@zeedhi/core';
import {
    Form, Button, TextInput, Text, Grid, Tabs,
    Modal, ModalService, AlertService, LoadingService, DialogService, Select, Textarea,
} from '@zeedhi/common';
import {
    Utilities,
} from '@/util/Utilities';
import { TekGrid } from '@zeedhi/teknisa-components-common';

export class SmNext00006000servicosController {
    public gridVisivel : boolean = true;

    private modalServico!: Modal;

    public modalRelacServico!: Modal;

    private headerFooterModal!: Modal;

    public currentRow : any = {};

    get formVisivel() {
        return !this.gridVisivel;
    }

    public openPreview({ row }: any) {
        this.gridVisivel = false;
        this.currentRow = row;
        this.reloadTabs();
    }

    public voltarGrid() {
        this.gridVisivel = true;
    }

    // Modal para adicionar Habilidades

    public async createModalHabilidade() {
        this.headerFooterModal = await ModalService.createFromJson('Service/Modal/sm_next_00006_001_servicos_modal_habilidades', true);
    }

    public destroyModalHabilidade() {
        this.headerFooterModal?.destroy();
    }

    public abrirModalHabilidade() {
        this.headerFooterModal.show();
    }

    public closeModalHabilidades() {
        this.headerFooterModal.hide();
    }

    // Modal para adicionar Realacionamento de Serviço

    public async createModalRelacServico() {
        this.modalRelacServico = await ModalService.createFromJson('Service/Modal/sm_next_00006_002_servicos_modal_relacionamento', true);
    }

    public destroyModalRelacServico() {
        this.modalRelacServico?.destroy();
    }

    public abrirModalRelacServico() {
        this.modalRelacServico.show();
    }

    public closeModalRelacServico() {
        this.modalRelacServico.hide();
    }

    // Modal para adicionar Serviço

    public async abrirModalInsercaoServico() {
        this.modalServico = await ModalService.createFromJson('Service/Modal/sm_next_00006_000_servicos_modal', true);
        this.modalServico.show();
    }

    public closeModalServico() {
        this.modalServico.hide();
        this.modalServico?.destroy();
    }

    public montarAbaDetalhes() {
        const footer = Metadata.getInstance<Form>('footer-servico');
        footer.isVisible = true;
        
        Metadata.getInstance<Select>('IDATIVO').value = this.currentRow.IDATIVO;
        Metadata.getInstance<TextInput>('NMSERVORCA').value = this.currentRow.NMSERVORCA;
        Metadata.getInstance<TextInput>('CDSERVORCA').value = this.currentRow.CDSERVORCA;
        Metadata.getInstance<TextInput>('CDPRODFATSERV').value = this.currentRow.CDPRODFATSERV;
        Metadata.getInstance<TextInput>('DSSERVORCA').value = this.currentRow.DSSERVORCA;
        Metadata.getInstance<TextInput>('DSSERVORCANF').value = this.currentRow.DSSERVORCANF;
        Metadata.getInstance<TextInput>('DSSERVORCABOL').value = this.currentRow.DSSERVORCABOL;
        Metadata.getInstance<Text>('NomeServico').text = this.currentRow.NMSERVORCA;
    }

    public coletarDadosInsercaoServico() {
        const formServico = Metadata.getInstance<Form>('formModalServicos').validate();
        const grid = Metadata.getInstance<TekGrid>('tek-grid-servicos');

        if (formServico) {
            LoadingService.show();
            const params = {
                IDATIVO: Metadata.getInstance<Select>('MODAL_IDATIVO').value,
                NMSERVORCA: Metadata.getInstance<TextInput>('MODAL_NMSERVORCA').value,
                CDPRODFATSERV: Metadata.getInstance<Select>('MODAL_CDPRODUTO').value,
                DSSERVORCA: Metadata.getInstance<Textarea>('MODAL_DSSERVORCA').value,
                DSSERVORCANF: Metadata.getInstance<Textarea>('MODAL_DSSERVORCANF').value,
                DSSERVORCABOL: Metadata.getInstance<Textarea>('MODAL_DSSERVORCABOL').value,
                CDSERVORCA: Metadata.getInstance<TextInput>('MODAL_CDSERVORCA').value,
            };
            Utilities.salvarDados('/insertServico', params).then(() => {
                Utilities.showSuccessMessage();
                grid.datasource.get();
                this.closeModalServico();
            }).catch((error : any) => {
                Utilities.showErrorMessage(error);
            }).finally(() => {
                LoadingService.hide();
            });
        } else {
            Utilities.alertMessage('Preencha todos os campos obrigatórios (*)!', 'error');
        }
    }

    public coletarDadosUpdateServico() {
        const formServico = Metadata.getInstance<Form>('form-ServicoForm').validate();
        const grid = Metadata.getInstance<TekGrid>('tek-grid-servicos');

        if (formServico) {
            LoadingService.show();
            const params = {
                IDATIVO: Metadata.getInstance<Select>('IDATIVO').value,
                NMSERVORCA: Metadata.getInstance<TextInput>('NMSERVORCA').value,
                CDPRODFATSERV: Metadata.getInstance<Select>('CDPRODFATSERV').value,
                DSSERVORCA: Metadata.getInstance<Textarea>('DSSERVORCA').value,
                DSSERVORCANF: Metadata.getInstance<Textarea>('DSSERVORCANF').value,
                DSSERVORCABOL: Metadata.getInstance<Textarea>('DSSERVORCABOL').value,
                CDSERVORCA: Metadata.getInstance<TextInput>('CDSERVORCA').value,
            };
            Utilities.salvarDados('/insertServico', params).then(() => {
                this.currentRow.IDATIVO = params.IDATIVO;
                this.currentRow.NMSERVORCA = params.NMSERVORCA;
                this.currentRow.CDPRODFATSERV = params.CDPRODFATSERV;
                this.currentRow.DSSERVORCA = params.DSSERVORCA;
                this.currentRow.DSSERVORCANF = params.DSSERVORCANF;
                this.currentRow.DSSERVORCABOL = params.DSSERVORCABOL;
                this.montarAbaDetalhes();
                grid.datasource.get();
            }).catch((error : any) => {
                Utilities.showErrorMessage(error);
            }).finally(() => {
                LoadingService.hide();
            });
        } else {
            Utilities.alertMessage('Preencha todos os campos obrigatórios (*)!', 'error');
        }
    }

    public dialogExcluirServico({ event }: any) {
        Utilities.showConfirmDeleteMessage(event, 'SmNext00006000servicosController.deletarServicos');
    }

    public deletarServicos() {
        const grid = Metadata.getInstance<TekGrid>('tek-grid-servicos');
        LoadingService.show();
        Http.post('/deletarServicos', {
            CDSERVORCA: (grid.selectedRows.map((element: any) => element.CDSERVORCA)),
        }).then(() => {
            grid.datasource.get();
            Utilities.showSuccessMessage();
        }).catch((error: any) => {
            Utilities.showErrorMessage(error);
        }).finally(() => {
            LoadingService.hide();
            DialogService.hide();
        });
    }

    public excluirRelacServico() {
        const params = Metadata.getInstance<any>('gridRelServico').selectedRows;
        LoadingService.show('Excluindo Relacionamento');
        Http.post('/deleteRelacServico', params).then(() => {
            this.atualizarGrid('gridRelServico');
            LoadingService.hide();
        }).catch((error: any) => {
            LoadingService.hide();
            Utilities.showErrorMessage(error);
        });
    }

    public alertMessage(message: string, type: string) {
        AlertService.show({
            name: 'basic-alert',
            text: message,
            top: true,
            left: false,
            timeout: 3000,
            color: type,
        });
    }

    public openFilter({ component }: any) {
        const filterButton = Metadata.getInstance<Button>(`${component.name}_filterButton`);
        filterButton.click({} as Event, {} as HTMLElement);
    }

    public atualizarGrid(grid :any) {
        const filterButton = Metadata.getInstance<Button>(`${grid}_refreshButton`);
        filterButton.click({} as Event, {} as HTMLElement);
    }

    public expandir({ component }: any) {
        const form = Metadata.getInstance<Form>(component.name.split('_')[1]);
        form.isVisible = !form.isVisible;
        component.iconName = (component.iconName === 'mdi-chevron-down') ? 'mdi-chevron-right' : 'mdi-chevron-down';
    }

    public salvarHabilidades() {
        const parametros = {
            CDSERVORCA: Metadata.getInstance<TextInput>('CDSERVORCA').value,
            CDHABILITSERV: Metadata.getInstance<TextInput>('CDHABILITSERV').value,
        };
        LoadingService.show('Adicionando Habilidade');
        Http.post('/validaHabilidades', parametros).then((sucesso) => {
            if (sucesso.data) {
                Http.post('/inserirRelacaoHabilidade', parametros).then(() => {
                    this.closeModalHabilidades();
                    this.atualizarGrid('gridHabilidade');
                    LoadingService.hide();
                }).catch((error: any) => {
                    LoadingService.hide();
                    Utilities.showErrorMessage(error);
                });
            } else {
                LoadingService.hide();
                DialogService.show({
                    name: 'basic-dialog',
                    type: 'error',
                    title: 'Operação bloqueada',
                    text: 'Uma ou mais habilidades selecionadas já estão cadastradas!',
                });
            }
        }).catch((error: any) => {
            LoadingService.hide();
            Utilities.showErrorMessage(error);
        });
    }

    public removerRelacaoHabilidade() {
        const params = Metadata.getInstance<any>('gridHabilidade').selectedRows;
        LoadingService.show('Excluindo Habilidade');
        Http.post('/removeRelacaoHabilidade', params).then(() => {
            this.atualizarGrid('gridHabilidade');
            LoadingService.hide();
        }).catch((error: any) => {
            LoadingService.hide();
            Utilities.showErrorMessage(error);
        });
    }

    public reloadTabs() {
        const setOfTabs = Metadata.getInstance<Tabs>('tabsServicoDetalhes');

        if (setOfTabs.tabs[setOfTabs.activeTab].name === 'tabServico') {
            this.montarAbaDetalhes();
        } else if (setOfTabs.tabs[setOfTabs.activeTab].name === 'tabRelServico') {
            this.montaRelacServico();
        } else if (setOfTabs.tabs[setOfTabs.activeTab].name === 'tabHabilidades') {
            this.montaFiltroHabilidade();
        }
    }

    public montaFiltroHabilidade() {
        const footer = Metadata.getInstance<Form>('footer-servico');
        footer.isVisible = false;
        const CDSERVORCA = Metadata.getInstance<TextInput>('CDSERVORCA').value;
        const grid = Metadata.getInstance<Grid>('gridHabilidade');
        this.setFilterGrid(grid, CDSERVORCA);
    }

    public montaRelacServico() {
        const footer = Metadata.getInstance<Form>('footer-servico');
        footer.isVisible = false;
        const CDSERVORCA = Metadata.getInstance<TextInput>('CDSERVORCA').value;
        const grid = Metadata.getInstance<Grid>('gridRelServico');
        this.setFilterGrid(grid, CDSERVORCA);
    }

    public setFilterGrid(grid: any, NR: any) {
        grid.datasource.setFilter({
            CDSERVORCA: NR,
        });
        grid.datasource.get();
    }

    public validaServicoRelacionado() {
        const params = {
            CDSERVORCARELAC: Metadata.getInstance<Form>('formModalRelacServico').value.CDSERVORCA,
            CDTPMEDIDASERV: Metadata.getInstance<Form>('formModalRelacServico').value.CDTPMEDIDASERV,
            QTMEDIDASERV: Metadata.getInstance<Form>('formModalRelacServico').value.QTMEDIDASERV,
            CDSERVORCA: this.currentRow.CDSERVORCA,
        };
        LoadingService.show();
        Http.post('/validaServicoRelacionado', params).then((response) => {
            if (response.data.VALIDA === false) {
                Http.post('/insertRelacServico', params).then(() => {
                    this.closeModalRelacServico();
                    this.atualizarGrid('gridRelServico');
                }).catch((error: any) => {
                    Utilities.showErrorMessage(error);
                }).finally(() => {
                    LoadingService.hide();
                });
            } else {
                AlertService.show({
                    top: true,
                    right: true,
                    color: 'error',
                    name: 'basic-alert',
                    text: 'Operação Bloqueada. Alguns dos serviços selecionados já estão relacionados para a medida informada.',
                });
            }
            LoadingService.hide();
        }).catch((error: any) => {
            LoadingService.hide();
            Utilities.showErrorMessage(error);
        });
    }
}
